@@ -16,12 +16,11 @@
  */
 
 /*
- * $Id: InvalidCastException.hpp 568078 2007-08-21 11:43:25Z amassari $
+ * $Id$
  */
 
-
-#if !defined(INVALIDCASTEXCEPTION_HPP)
-#define INVALIDCASTEXCEPTION_HPP
+#if !defined(XERCESC_INCLUDE_GUARD_INVALIDCASTEXCEPTION_HPP)
+#define XERCESC_INCLUDE_GUARD_INVALIDCASTEXCEPTION_HPP
 
 #include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/XMLException.hpp>
