@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -16,17 +16,18 @@
  */
 
 /*
- * $Id: TransService.hpp 568078 2007-08-21 11:43:25Z amassari $
+ * $Id$
  */
 
-#ifndef TRANSSERVICE_HPP
-#define TRANSSERVICE_HPP
+#if !defined(XERCESC_INCLUDE_GUARD_TRANSSERVICE_HPP)
+#define XERCESC_INCLUDE_GUARD_TRANSSERVICE_HPP
 
 #include <xercesc/util/XMemory.hpp>
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/framework/XMLRecognizer.hpp>
 #include <xercesc/util/RefHashTableOf.hpp>
 #include <xercesc/util/RefVectorOf.hpp>
+#include <xercesc/util/Janitor.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
@@ -82,7 +83,7 @@ public :
     (
         const   XMLCh* const            encodingName
         ,       XMLTransService::Codes& resValue
-        , const unsigned int            blockSize
+        , const XMLSize_t               blockSize
         , MemoryManager* const          manager = XMLPlatformUtils::fgMemoryManager
     );
 
@@ -90,7 +91,7 @@ public :
     (
         const   char* const             encodingName
         ,       XMLTransService::Codes& resValue
-        , const unsigned int            blockSize
+        , const XMLSize_t               blockSize
         , MemoryManager* const          manager = XMLPlatformUtils::fgMemoryManager
     );
 
@@ -98,7 +99,7 @@ public :
     (
         XMLRecognizer::Encodings        encodingEnum
         ,       XMLTransService::Codes& resValue
-        , const unsigned int            blockSize
+        , const XMLSize_t               blockSize
         , MemoryManager* const          manager = XMLPlatformUtils::fgMemoryManager
     );
 
@@ -116,30 +117,33 @@ public :
     (
         const   XMLCh* const    comp1
         , const XMLCh* const    comp2
-        , const unsigned int    maxChars
+        , const XMLSize_t       maxChars
     ) = 0;
 
     virtual const XMLCh* getId() const = 0;
 
-    virtual bool isSpace(const XMLCh toCheck) const = 0;
-
-    virtual XMLLCPTranscoder* makeNewLCPTranscoder() = 0;
+    // -----------------------------------------------------------------------
+    //    Create a new transcoder for the local code page.
+    //
+    //  @param manager The memory manager to use.
+    // -----------------------------------------------------------------------
+    virtual XMLLCPTranscoder* makeNewLCPTranscoder(MemoryManager* manager) = 0;
 
     virtual bool supportsSrcOfs() const = 0;
 
-    virtual void upperCase(XMLCh* const toUpperCase) const = 0;
-    virtual void lowerCase(XMLCh* const toLowerCase) const = 0;
+    virtual void upperCase(XMLCh* const toUpperCase) = 0;
+    virtual void lowerCase(XMLCh* const toLowerCase) = 0;
 
-	// -----------------------------------------------------------------------
-    //	Allow users to add their own encodings to the intrinsinc mapping
-	//	table
-	//	Usage:
-	//		XMLTransService::addEncoding (
-	//			gMyEncodingNameString
-    //			, new ENameMapFor<MyTransClassType>(gMyEncodingNameString)
-	//		);
     // -----------------------------------------------------------------------
-	static void addEncoding(const XMLCh* const encoding, ENameMap* const ownMapping);
+    //    Allow users to add their own encodings to the intrinsic mapping
+    //    table
+    //    Usage:
+    //        XMLTransService::addEncoding (
+    //            gMyEncodingNameString
+    //            , new ENameMapFor<MyTransClassType>(gMyEncodingNameString)
+    //        );
+    // -----------------------------------------------------------------------
+    static void addEncoding(const XMLCh* const encoding, ENameMap* const ownMapping);
 
 
 protected :
@@ -159,7 +163,7 @@ protected :
     (
         const   XMLCh* const            encodingName
         ,       XMLTransService::Codes& resValue
-        , const unsigned int            blockSize
+        , const XMLSize_t               blockSize
         , MemoryManager* const          manager
     ) = 0;
 
@@ -196,21 +200,10 @@ private :
     // -----------------------------------------------------------------------
     void strictIANAEncoding(const bool newState);
     bool isStrictIANAEncoding();
-    static void reinitMappings();
-    static void reinitMappingsRecognizer();
 
+    friend class XMLInitializer;
 };
 
-
-
-/**
-  * <code>DOMString</code> is the generic string class that stores all strings
-  * used in the DOM C++ API.
-  *
-  * Though this class supports most of the common string operations to manipulate
-  * strings, it is not meant to be a comphrehensive string class.
-  */
-
 /**
   *   <code>XMLTranscoder</code> is for transcoding non-local code
   *   page encodings, i.e.  named encodings. These are used internally
@@ -223,29 +216,29 @@ class XMLUTIL_EXPORT XMLTranscoder : public XMemory
 {
 public :
 
-	/**
-	 * This enum is used by the <code>transcodeTo()</code> method
-	 * to indicate how to react to unrepresentable characters. The
-	 * <code>transcodeFrom()</code> method always works the
-	 * same. It will consider any invalid data to be an error and
-	 * throw.
-	 */
+    /**
+     * This enum is used by the <code>transcodeTo()</code> method
+     * to indicate how to react to unrepresentable characters. The
+     * <code>transcodeFrom()</code> method always works the
+     * same. It will consider any invalid data to be an error and
+     * throw.
+     */
     enum UnRepOpts
     {
-        UnRep_Throw		/**< Throw an exception */
-        , UnRep_RepChar		/**< Use the replacement char */
+        UnRep_Throw        /**< Throw an exception */
+        , UnRep_RepChar        /**< Use the replacement char */
     };
 
 
-	/** @name Destructor. */
-	//@{
+    /** @name Destructor. */
+    //@{
 
-	 /**
-	  * Destructor for XMLTranscoder
-	  *
-	  */
+     /**
+      * Destructor for XMLTranscoder
+      *
+      */
     virtual ~XMLTranscoder();
-	//@}
+    //@}
 
 
 
@@ -271,13 +264,13 @@ public :
       */
 
 
-    virtual unsigned int transcodeFrom
+    virtual XMLSize_t transcodeFrom
     (
         const   XMLByte* const          srcData
-        , const unsigned int            srcCount
+        , const XMLSize_t               srcCount
         ,       XMLCh* const            toFill
-        , const unsigned int            maxChars
-        ,       unsigned int&           bytesEaten
+        , const XMLSize_t               maxChars
+        ,       XMLSize_t&              bytesEaten
         ,       unsigned char* const    charSizes
     ) = 0;
 
@@ -294,13 +287,13 @@ public :
       * @return Returns the number of chars put into the target buffer
       */
 
-    virtual unsigned int transcodeTo
+    virtual XMLSize_t transcodeTo
     (
         const   XMLCh* const    srcData
-        , const unsigned int    srcCount
+        , const XMLSize_t       srcCount
         ,       XMLByte* const  toFill
-        , const unsigned int    maxBytes
-        ,       unsigned int&   charsEaten
+        , const XMLSize_t       maxBytes
+        ,       XMLSize_t&      charsEaten
         , const UnRepOpts       options
     ) = 0;
 
@@ -312,7 +305,7 @@ public :
     virtual bool canTranscodeTo
     (
         const   unsigned int    toCheck
-    )   const = 0;
+    ) = 0;
 
     //@}
 
@@ -323,7 +316,7 @@ public :
      *
        * @return The block size indicated in the constructor.
        */
-    unsigned int getBlockSize() const;
+    XMLSize_t getBlockSize() const;
 
     /** Get the encoding name
       *
@@ -331,7 +324,7 @@ public :
       *    <code>XMLTranscoder</code> object is for
       */
     const XMLCh* getEncodingName() const;
-	//@}
+    //@}
 
     /** @name Getter methods*/
     //@{
@@ -345,7 +338,7 @@ public :
       */
     MemoryManager* getMemoryManager() const;
 
-	//@}
+    //@}
 
 protected :
     // -----------------------------------------------------------------------
@@ -354,7 +347,7 @@ protected :
     XMLTranscoder
     (
         const   XMLCh* const    encodingName
-        , const unsigned int    blockSize
+        , const XMLSize_t       blockSize
         , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     );
 
@@ -362,11 +355,6 @@ protected :
     // -----------------------------------------------------------------------
     //  Protected helper methods
     // -----------------------------------------------------------------------
-    // As the body of this function is commented out it could be removed.
-    // However, currently all calls to it are guarded by #if defined(XERCES_DEBUG)
-    // so will leave it for now.
-    void checkBlockSize(const unsigned int toCheck);
-
 
 private :
     // -----------------------------------------------------------------------
@@ -385,7 +373,7 @@ private :
     //      This is the name of the encoding this encoder is for. All basic
     //      XML transcoder's are for named encodings.
     // -----------------------------------------------------------------------
-    unsigned int    fBlockSize;
+    XMLSize_t       fBlockSize;
     XMLCh*          fEncodingName;
     MemoryManager*  fMemoryManager;
 };
@@ -416,25 +404,36 @@ public :
     //          its assumed that the buffer is physically one char or byte
     //          larger.
     // -----------------------------------------------------------------------
-    virtual unsigned int calcRequiredSize(const char* const srcText
-        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) = 0;
 
-    virtual unsigned int calcRequiredSize(const XMLCh* const srcText
-        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) = 0;
-
-    virtual char* transcode(const XMLCh* const toTranscode) = 0;
+    // -----------------------------------------------------------------------
+    //  The 'normal' way to transcode a XMLCh-string from/to local string
+    //  representation
+    //
+    //  NOTE: Both methods return a string allocated via the MemoryManager.
+    //        It is the responsibility of the calling environment to
+    //        release this string after use.
+    // -----------------------------------------------------------------------
     virtual char* transcode(const XMLCh* const toTranscode,
-                            MemoryManager* const manager) = 0;
+                            MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) = 0;
 
-    virtual XMLCh* transcode(const char* const toTranscode) = 0;
     virtual XMLCh* transcode(const char* const toTranscode,
-                             MemoryManager* const manager) = 0;
+                             MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) = 0;
+
+
+    // -----------------------------------------------------------------------
+    //  DEPRECATED old transcode interface
+    // -----------------------------------------------------------------------
+    virtual XMLSize_t calcRequiredSize(const char* const srcText
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) = 0;
+
+    virtual XMLSize_t calcRequiredSize(const XMLCh* const srcText
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager) = 0;
 
     virtual bool transcode
     (
         const   char* const     toTranscode
         ,       XMLCh* const    toFill
-        , const unsigned int    maxChars
+        , const XMLSize_t       maxChars
         , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     ) = 0;
 
@@ -442,7 +441,7 @@ public :
     (
         const   XMLCh* const    toTranscode
         ,       char* const     toFill
-        , const unsigned int    maxBytes
+        , const XMLSize_t       maxBytes
         , MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager
     ) = 0;
 
@@ -462,6 +461,187 @@ private :
     XMLLCPTranscoder& operator=(const XMLLCPTranscoder&);
 };
 
+//
+// This class can be used to transcode to a target encoding. It manages the
+// memory allocated for the transcode in an exception safe manner, automatically
+// deleting it when the class goes out of scope.
+//
+class XMLUTIL_EXPORT TranscodeToStr
+{
+public:
+    // -----------------------------------------------------------------------
+    //  Public constructors and destructor
+    // -----------------------------------------------------------------------
+
+    /** Converts from the internal XMLCh* encoding to the specified encoding
+      *
+      * @param in       the null terminated source buffer to be transcoded
+      * @param encoding the name of the encoding to transcode to
+      * @param manager  the memory manager to use
+      */
+    TranscodeToStr(const XMLCh *in, const char *encoding,
+                   MemoryManager *manager = XMLPlatformUtils::fgMemoryManager);
+
+    /** Converts from the internal XMLCh* encoding to the specified encoding
+      *
+      * @param in       the source buffer to be transcoded
+      * @param length   the length of the source buffer
+      * @param encoding the name of the encoding to transcode to
+      * @param manager  the memory manager to use
+      */
+    TranscodeToStr(const XMLCh *in, XMLSize_t length, const char *encoding,
+                   MemoryManager *manager = XMLPlatformUtils::fgMemoryManager);
+
+    /** Converts from the internal XMLCh* encoding to the specified encoding
+      *
+      * @param in       the null terminated source buffer to be transcoded
+      * @param trans    the transcoder to use
+      * @param manager  the memory manager to use
+      */
+    TranscodeToStr(const XMLCh *in, XMLTranscoder* trans,
+                   MemoryManager *manager = XMLPlatformUtils::fgMemoryManager);
+
+    /** Converts from the internal XMLCh* encoding to the specified encoding
+      *
+      * @param in       the source buffer to be transcoded
+      * @param length   the length of the source buffer
+      * @param trans    the transcoder to use
+      * @param manager  the memory manager to use
+      */
+    TranscodeToStr(const XMLCh *in, XMLSize_t length, XMLTranscoder* trans,
+                   MemoryManager *manager = XMLPlatformUtils::fgMemoryManager);
+
+    ~TranscodeToStr();
+
+    /** @name Getter methods */
+    //@{
+
+    /** Returns the transcoded, null terminated string
+      * @return the transcoded string
+      */
+    const XMLByte *str() const;
+
+    /** Returns the transcoded, null terminated string - adopting
+      * the memory allocated to it from the TranscodeToStr object
+      * @return the transcoded string
+      */
+    XMLByte *adopt();
+
+    /** Returns the length of the transcoded string in bytes. The length
+      * does not include the null terminator.
+      * @return the length of the transcoded string in bytes
+      */
+    XMLSize_t length () const;
+
+    //@}
+
+private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    TranscodeToStr(const TranscodeToStr &);
+    TranscodeToStr &operator=(const TranscodeToStr &);
+
+    // -----------------------------------------------------------------------
+    //  Private helper methods
+    // -----------------------------------------------------------------------
+    void transcode(const XMLCh *in, XMLSize_t len, XMLTranscoder* trans);
+
+    // -----------------------------------------------------------------------
+    //  Private data members
+    //
+    //  fString
+    //      The transcoded string
+    //
+    //  fBytesWritten
+    //      The length of the transcoded string in bytes
+    // -----------------------------------------------------------------------
+    ArrayJanitor<XMLByte> fString;
+    XMLSize_t fBytesWritten;
+    MemoryManager *fMemoryManager;
+};
+
+//
+// This class can be used to transcode from a source encoding. It manages the
+// memory allocated for the transcode in an exception safe manner, automatically
+// deleting it when the class goes out of scope.
+//
+class XMLUTIL_EXPORT TranscodeFromStr
+{
+public:
+    // -----------------------------------------------------------------------
+    //  Public constructors and destructor
+    // -----------------------------------------------------------------------
+
+    /** Converts from the specified encoding to the internal XMLCh* encoding
+      *
+      * @param data     the source buffer to be transcoded
+      * @param length   the length of the source buffer
+      * @param encoding the name of the encoding to transcode to
+      * @param manager  the memory manager to use
+      */
+    TranscodeFromStr(const XMLByte *data, XMLSize_t length, const char *encoding,
+                     MemoryManager *manager = XMLPlatformUtils::fgMemoryManager);
+
+    /** Converts from the specified encoding to the internal XMLCh* encoding
+      *
+      * @param data     the source buffer to be transcoded
+      * @param length   the length of the source buffer
+      * @param trans    the transcoder to use
+      * @param manager  the memory manager to use
+      */
+    TranscodeFromStr(const XMLByte *data, XMLSize_t length, XMLTranscoder *trans,
+                     MemoryManager *manager = XMLPlatformUtils::fgMemoryManager);
+
+    ~TranscodeFromStr();
+
+    /** @name Getter methods */
+    //@{
+
+    /** Returns the transcoded, null terminated string
+      * @return the transcoded string
+      */
+    const XMLCh *str() const;
+
+    /** Returns the transcoded, null terminated string - adopting
+      * the memory allocated to it from the TranscodeFromStr object
+      * @return the transcoded string
+      */
+    XMLCh *adopt();
+
+    /** Returns the length of the transcoded string in characters. The length
+      * does not include the null terminator.
+      * @return the length of the transcoded string in characters
+      */
+    XMLSize_t length() const;
+
+    //@}
+
+private:
+    // -----------------------------------------------------------------------
+    //  Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    TranscodeFromStr(const TranscodeFromStr &);
+    TranscodeFromStr &operator=(const TranscodeFromStr &);
+
+    // -----------------------------------------------------------------------
+    //  Private helper methods
+    // -----------------------------------------------------------------------
+    void transcode(const XMLByte *in, XMLSize_t length, XMLTranscoder *trans);
+
+    // -----------------------------------------------------------------------
+    //  Private data members
+    //
+    //  fString
+    //      The transcoded string
+    //
+    //  fCharsWritten
+    //      The length of the transcoded string in characters
+    // -----------------------------------------------------------------------
+    ArrayJanitor<XMLCh> fString;
+    XMLSize_t fCharsWritten;
+    MemoryManager *fMemoryManager;
+};
 
 // ---------------------------------------------------------------------------
 //  XMLTranscoder: Getter methods
@@ -474,7 +654,7 @@ inline MemoryManager* XMLTranscoder::getMemoryManager() const
 // ---------------------------------------------------------------------------
 //  XMLTranscoder: Protected helper methods
 // ---------------------------------------------------------------------------
-inline unsigned int XMLTranscoder::getBlockSize() const
+inline XMLSize_t XMLTranscoder::getBlockSize() const
 {
     return fBlockSize;
 }
@@ -484,6 +664,44 @@ inline const XMLCh* XMLTranscoder::getEncodingName() const
     return fEncodingName;
 }
 
+// ---------------------------------------------------------------------------
+//  TranscodeToStr: Getter methods
+// ---------------------------------------------------------------------------
+inline const XMLByte *TranscodeToStr::str() const
+{
+    return fString.get();
+}
+
+inline XMLByte *TranscodeToStr::adopt()
+{
+    fBytesWritten = 0;
+    return fString.release();
+}
+
+inline XMLSize_t TranscodeToStr::length () const
+{
+    return fBytesWritten;
+}
+
+// ---------------------------------------------------------------------------
+//  TranscodeFromStr: Getter methods
+// ---------------------------------------------------------------------------
+inline const XMLCh *TranscodeFromStr::str() const
+{
+    return fString.get();
+}
+
+inline XMLCh *TranscodeFromStr::adopt()
+{
+    fCharsWritten = 0;
+    return fString.release();
+}
+
+inline XMLSize_t TranscodeFromStr::length() const
+{
+    return fCharsWritten;
+}
+
 XERCES_CPP_NAMESPACE_END
 
 #endif
